#ifndef _RESOLUTION_H
#define _RESOLUTION_H

#include <arbre.h>

#include <stdlib.h>
#include <string.h>

typedef struct {
    char ** mots;
    int buf;
    int nbmots;
} ListMots;

/*   resolution
 * Retourne une liste de mots contenant tous les mots rÃ©alisables dans le
 * worddle passÃ© en argument. La liste de mots peut contenir des doublons et
 * n'est pas ordonnÃ©e.
 */
ListMots * resolution(Arbre dico, char ** worddle, int taille);
/*   mots
 * Cherche récursivement des mots dans le worddle et les ajoute à la liste.
 *  - taille est la taille du worddle
 *  - i et j sont la position dans la lecture du worddle 
 *  - visites est un tableau de booléens contenant si oui ou non un noeud
 *    a déjà été visité.
 *  - pref est le préfixe du mot pouvant exister.
 *  - p est la position de la prochaine lettre à ajouter dans pref
 */
void mots(ListMots * solution, char ** worddle, int taille, int i, int j,
	Arbre dico, char ** visites, char pref[], int p);


/*   ajouteMot
 * Ajoute la chaine de caractères mot à la liste de mots pointée. Agrandit
 * la quantité de mots enregistrables si nécessaire.
 */
int ajouteMot(ListMots * liste, char * mot);
/*   libererMots
 * Libère la liste de mots de la mémoire
 */
void libererMots(ListMots * liste);


/*    supprime_doublons
 * Supprime les doublons d'une liste de mots _ordonnée_.
 */
void supprime_doublons(ListMots * liste);

/*   tri
 * Tri les n listes de mots consécutives
 */
void tri(ListMots * solution, int n);

/*   affiche
 * Affiche n liste de mots consécutives par ordre décroissant de longueur
 * ( la longueur correspondant à la position de la liste )
 */
void affiche(ListMots * solution, int n);

#endif
