#include <arbre.h>

/*   creerFeuille
 * Retourne un pointeur vers un noeud de l'arbre lexicographique contenant
 * le caractre pass en argument et n'ayant pas de fils ni de frre.
 * Retourne NULL si la cration de noeud choue.
 */
Noeud * creerFeuille(char c)
{
    Noeud * tmp = NULL;
    
    tmp = calloc(sizeof(Noeud), 1);
    if ( tmp == NULL )
	return NULL;
    
    tmp->l = c;

    return tmp;
}

/*   detruireArbre
 * Libre l'arbre lxicographique pass en argument de la mmoire.
 */
void detruireArbre(Arbre a)
{
    if ( a != NULL ) {
	detruireArbre(a->frere);
	detruireArbre(a->fils);
	free(a);
    }
}

/*   insertion
 * Insre le mot pass en argument dans l'arbre lexicographique point.
 * Retourne 1 si l'insertion est russie, 0 sinon.
 */
int insertion(Arbre * a, char * mot)
{
    if ( *a == NULL ) {
	*a = creerFeuille(*mot);
	if ( *a == NULL )
	    return 0;

	if ( *mot == 0 )
	    return 1;
	else
	    return insertion(&((*a)->fils), mot+1);
    }

    if ( (*a)->l == *mot ) {
	if ( *mot == 0 )
	    return 1;
	else
	    return insertion(&((*a)->fils), mot+1);
    } else if ( (*a)->l > *mot ) {
	Arbre tmp = *a;
	*a = creerFeuille(*mot);
	if ( *a == NULL )
	    return 0;
	(*a)->frere = tmp;

	if ( *mot == 0 )
	    return 1;
	else
	    return insertion(&((*a)->fils), mot+1);
    }

    return insertion(&((*a)->frere), mot);
}

/*   lecture
 * Lit le fichier pass en argument et ajoute chaque ligne  l'arbre
 * lexicographique donn.
 */
int lecture(FILE * f, Arbre * a)
{
    char * moulinette = "-\\|/";
    int i = 0 ;

    if ( a == NULL )
	return 0;

    char * mot = NULL;
    // taille correspond  la taille de la ligne lue par getline ( \0 compris )
    // tcache correspond  la taille du cache de getline ( ie. la zone alloue
    // par un malloc qu'utilise getline )
    int taille, tcache = 0;

    while ( (taille = getline(&mot, &tcache, f)) != -1 ) {
	// getline enregistre le \n lu en fin de ligne, on l'enlve.
	mot[taille-1] = 0;
	if ( ! insertion(a, mot) )
	    return 0;
	fprintf(stderr,"\r%c",moulinette[i]);
	i = (i+1)%4;
    }

    free(mot);

    return 1;
}

/*    chercheLettre
 * Retourne le sous arbre de c s'il existe parmi les frres de dico, NULL sinon
 */
Arbre chercheLettre(Arbre dico, char c)
{
    while ( dico != NULL ) {
	if ( dico->l == c )
	    return dico;
	dico = dico->frere;
    }

    return NULL;
}
