#include <resolution.h>

/*   ajouteMot
 * Ajoute la chaine de caractres mot  la liste de mots pointe. Agrandit
 * la quantit de mots enregistrables si ncessaire.
 */
int ajouteMot(ListMots * liste, char * mot)
{
    if ( liste->buf == liste->nbmots ) {
	liste->mots = realloc(liste->mots, (liste->buf+42) * sizeof(char *));
	liste->buf += 42;
	if ( liste->mots == NULL )
	    return 0;
    }

    liste->mots[liste->nbmots] = malloc(strlen(mot)+1);
    if ( liste->mots[liste->nbmots] == NULL )
	return 0;

    strcpy(liste->mots[liste->nbmots], mot);
    liste->nbmots++;

    return 1;
}

/*   libererMots
 * Libre la liste de mots de la mmoire
 */
void libererMots(ListMots * liste)
{
    if ( liste->buf != 0 ) {
	int i;
	for ( i=0 ; i < liste->nbmots ; i++ )
	    free(liste->mots[i]);
	free(liste->mots);
    }
}

/*   resolution
 * Retourne une liste de mots contenant tous les mots ralisables dans le
 * worddle pass en argument. La liste de mots peut contenir des doublons et
 * n'est pas ordonne.
 */
ListMots * resolution(Arbre dico, char ** worddle, int taille)
{
    ListMots * solution = calloc(taille*taille, sizeof(ListMots));
    char ** visites = NULL;
    char pref[taille*taille+1];
    int i,j;

    pref[0] = 0;

    // Initialisation du tableau de visites
    visites = calloc(taille, sizeof(char *));
    for ( i=0 ; i < taille ; i++ )
	visites[i] = calloc(taille, sizeof(char));

    // Recherche
    for ( i=0 ; i < taille ; i++ )
	for ( j=0 ; j < taille ; j++ )
	    mots(solution, worddle, taille, i, j, dico, visites, pref, 0);

    // Libration du tableau de visites
    for ( i=0 ; i < taille ; i++ )
	free(visites[i]);
    free(visites);

    return solution;
}

/*   mots
 * Cherche rcursivement des mots dans le worddle et les ajoute  la liste.
 *  - taille est la taille du worddle
 *  - i et j sont la position dans la lecture du worddle 
 *  - visites est un tableau de boolens contenant si oui ou non un noeud
 *    a dj t visit.
 *  - pref est le prfixe du mot pouvant exister.
 *  - p est la position de la prochaine lettre  ajouter dans pref
 */
void mots(ListMots * solution, char ** worddle, int taille, int i, int j,
	Arbre dico, char ** visites, char pref[], int p)
{
    Arbre sousArbre;
    int a, b;

    if ( (sousArbre = chercheLettre(dico, worddle[i][j])) != NULL ) {
	visites[i][j] = 1;
	pref[p] = worddle[i][j];

	/* Mot trouv ? */
	if ( sousArbre->fils->l == 0 ) {
	    pref[p+1] = 0;
	    ajouteMot(&solution[strlen(pref)], pref);
	}

	/* On cherche les nouvelles possibilits */
	for ( a = ( i-1 >= 0 ) ? i-1 : i
		; a <= i+1 && a < taille
		; a++ )
	    for ( b = ( j-1 >= 0 ) ? j-1 : j
		    ; b <= j+1 && b < taille
		    ; b++ )
		if ( ! visites[a][b] )
		    mots(solution, worddle, taille, a, b, sousArbre->fils,
			    visites, pref, p+1);

	visites[i][j] = 0;
    }
}

/*
 * Fonction de comparaison utilise par tri()
 */
int compar(const void * a, const void * b)
{
    return strcmp(*(char **)a, *(char **)b);
}

/*
 * Tri les n listes de mots conscutives
 */
void tri(ListMots * solution, int n)
{
    int i;

    for ( i=0 ; i < n ; i++ )
	if ( solution[i].nbmots > 1 ) {
	    qsort(solution[i].mots, solution[i].nbmots, sizeof(char *), compar);
	    supprime_doublons(&solution[i]);
	}
}

/*
 * Affiche n liste de mots conscutives par ordre dcroissant de longueur
 */
void affiche(ListMots * solution, int n)
{
    int i,j;

    for ( i=n-1 ; i >= 0 ; i-- )
	for ( j=0 ; j < solution[i].nbmots ; j++ )
	    if ( solution[i].mots[j] != NULL )
		puts(solution[i].mots[j]);
}

/*    supprime_doublons
 * Supprime les doublons d'une liste de mots _ordonne_.
 */
void supprime_doublons(ListMots * liste)
{
    if ( liste->nbmots < 2 )
	return;

    int i, j;
    for ( i=1, j=0 ; i < liste->nbmots ; i++ ) {
	if ( strcmp(liste->mots[j], liste->mots[i]) == 0 ) {
	    free(liste->mots[i]);
	    liste->mots[i] = NULL;
	} else {
	    do {
		j++;
	    } while ( liste->mots[j] == NULL );
	}
    }
}

