#include <worddle.h>

/*   lecture_worddle
 * Lit un worddle ( tableau de caractres de taille : taille * taille )
 * sur l'entre standard.
 */
void lecture_worddle(char ** worddle, int taille)
{
    int i,j,c;

    for ( i=0 ; i < taille ; i++ )
	for ( j=0 ; j < taille ; j++ )
	    do {
		c = getchar();
		worddle[i][j] = c;
	    } while ( c == ' ' || c == '\n' );
}

/*   init
 * Initialisation du programme :
 *  - lecture des arguments de la ligne de commande
 *  - cration du dictionnaire sous forme d'arbre lexicographique
 *  - lit un worddle sur l'entre standard
 */
void init(int argc, char ** argv, Arbre * dico, char *** worddle, int * taille)
{
    char * nom_fichier = NULL;
    FILE * fichier_dico = NULL;
    int c;

    // Analyse de la ligne de commande
    while ( (c = getopt(argc, argv, "d:s:")) != -1 ) {
	switch(c) {
	    case 'd':
		nom_fichier = malloc(strlen(optarg)+1);
		strcpy(nom_fichier, optarg);
		fichier_dico = fopen(nom_fichier, "r");
		break;
	    case 's':
		*taille = (int) strtol(optarg, NULL, 10);
		break;
	}
    }

    // Lecture du worddle
    if ( *taille > 0 ) {
	// Allocation mmoire pour le tableau 2D de caractres
	*worddle = malloc(sizeof(char *) * *taille);
	if ( *worddle == NULL ) {
	    fprintf(stderr, "Erreur fatale : Pas assez de mmoire pour "
		    "enregistrer le worddle\n.");
	    exit(11);
	} else {
	    for ( c=0 ; c < *taille ; c++ ) {
		(*worddle)[c] = malloc(sizeof(char) * *taille );
		if ( (*worddle)[c] == NULL ) {
		    fprintf(stderr, "Erreur fatale : Pas assez de mmoire pour"
			    " enregistrer le worddle\n");
		    for ( ; c >= 0 ; c-- )
			free((*worddle)[c]);
		    exit(11);
		}
	    }
	}
	// Lecture
	lecture_worddle(*worddle, *taille);
    } else {
	fprintf(stderr,
		"Erreur fatale : La taille du worddle doit tre suprieure "
		"0\n");
	free(nom_fichier);
	exit(10);
    }

    // Lecture du dictionnaire
    if ( fichier_dico != NULL ) {
	if ( lecture(fichier_dico, dico) == 0 ) {
	    fprintf(stderr,
		    "Erreur fatale : erreur  la cration du dictionnaire\n");
	    exit(2);
	}
	fclose(fichier_dico);
	free(nom_fichier);
    } else {
	fprintf(stderr,"Erreur fatale : Pas de dictionnaire\n");
	*taille = 0;
	if ( nom_fichier != NULL )
	    free(nom_fichier);
	exit(1);
    }
}

/*
 * Libre la mmoire utilise par le programme
 */
void liberation(char ** worddle, ListMots * solution, Arbre dico, int taille)
{
    int i;

    // Libration des mots
    for ( i=0 ; i < taille*taille ; i++ )
	libererMots(&(solution[i]));
    free(solution);

    // Libration du dictionnaire
    detruireArbre(dico);

    // Libration du worddle
    for ( i=0 ; i < taille ; i++ )
	free(worddle[taille]);
    free(worddle);
}

int main(int argc, char ** argv)
{
    Arbre dico = NULL;
    char ** worddle = NULL;
    int taille = 0;
    ListMots * solution = NULL;

    init(argc, argv, &dico, &worddle, &taille);

    solution = resolution(dico, worddle, taille);
    tri(solution, taille * taille);
    affiche(solution, taille * taille);

    liberation(worddle, solution, dico, taille);

    return 0;
}
